
//----------------------------------------------------------------------
// FileGetPath - copies a file's path (but not file name) to a new location
void FileGetPath( char *filename, char *buffer );

//----------------------------------------------------------------------
// FileGetName - copies a file's name (discarding path) and ext to new location
void FileGetName( char *filename, char *buffer );

//----------------------------------------------------------------------
// FileGetExt - returns pointer to extension within this filename
//            - if no extension, returns pointer to "\0" so still usable string
char *FileGetExt( char *filename );



//----------------------------------------------------------------------
// FileChangeExt - copy apply a new extension to a file name
//               - accepts .ext or ext
void FileChangeExt( const char *filename, const char *newext, char *buffer );

//----------------------------------------------------------------------
// FileExists - returns 1 if true
int FileExists( const char *filename );

//----------------------------------------------------------------------
// FileRenameExt - renames file to a given extension in same directory
int FileRenameExt( char *filename, char *newext );


